/*********************************************************************************

	SoundServer - Written by Arnaud Carr (aka Leonard / OXYGENE)
	Part of the "Leonard Homepage Articles".
	Read the complete article on my web page:
	
	  http://leonard.oxg.free.fr

	PART 1: Using WaveOut API.
	PART 2: Using DirectSound API.
	How to make a SoundServer under windows, to play various sound.

	this sample plays a 440Hz sin wav.

	arnaud.carre@freesurf.fr

*********************************************************************************/

#define	USE_DIRECTSOUND			0			// Set to 0 to test WaveOut API


#include <windows.h>
#include <math.h>

#if USE_DIRECTSOUND
#include "DXSoundServer.h"
#define	WND_TITLE		"DX-SoundServer"
#else
#include "SoundServer.h"
#define	WND_TITLE		"WaveOut SoundServer"
#endif

#define	WND_CLASSNAME	"SoundWindow"
#define	TWO_PI			(3.1415926f * 2.f)
#define	SIN_STEP		((TWO_PI * 440.f) / 44100.f)
static	float			sinPos = 0.f;

//-----------------------------------------------------------------------------
// Your sound server call back !
// Call what you want here !! An MP3 decoder, a classic MOD player
// or the YM-Engine lirary ! :-)
//
// For that test purpose, our rout computes a classic 440Hz sin wave.
//-----------------------------------------------------------------------------
void	mySoundProc(void *pSoundBuffer,long bufferLen)
{

		// Convert params, assuming we create a 16bits, mono waveform.
		signed short *pSample = (signed short*)pSoundBuffer;
		long	nbSample = bufferLen / sizeof(signed short);

		for (int i=0;i<nbSample;i++)
		{
			*pSample++ = (signed short)(16384.f * sin(sinPos));
			sinPos += SIN_STEP;
		}
		if (sinPos >= TWO_PI) sinPos -= TWO_PI;
}



LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{

	switch (message) 
	{
		case WM_DESTROY:
			PostQuitMessage(0);
			break;

		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
   }
   return 0;
}





HWND	windowCreate(HINSTANCE hInstance)
{
WNDCLASSEX wcex;


		wcex.cbSize = sizeof(WNDCLASSEX); 
		wcex.style			= CS_HREDRAW | CS_VREDRAW;
		wcex.lpfnWndProc	= (WNDPROC)WndProc;
		wcex.cbClsExtra		= 0;
		wcex.cbWndExtra		= 0;
		wcex.hInstance		= hInstance;
		wcex.hIcon			= NULL;
		wcex.hCursor		= CopyCursor(LoadCursor(NULL, IDC_ARROW));
		wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
		wcex.lpszMenuName	= NULL;
		wcex.lpszClassName	= WND_CLASSNAME;
		wcex.hIconSm		= NULL;
		RegisterClassEx(&wcex);

		HWND hWnd = CreateWindow(	WND_CLASSNAME,WND_TITLE,WS_OVERLAPPEDWINDOW,
									CW_USEDEFAULT,CW_USEDEFAULT,
									320,200, 
									NULL, NULL, hInstance, NULL);

		if (!hWnd) return NULL;
		ShowWindow(hWnd, SW_SHOWNORMAL);
		UpdateWindow(hWnd);
		return hWnd;

}




int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
 	

		HWND hWnd = windowCreate(hInstance);
		if (hWnd)
		{

		// Init our SoundServer
#if USE_DIRECTSOUND
			CDXSoundServer *pServer = new CDXSoundServer(hWnd);
#else
			CSoundServer *pServer = new CSoundServer;
#endif

			if (pServer->open(mySoundProc))
			{

				// Enter the classic window message pump.
				BOOL bGotMsg;
				MSG	msg;
				PeekMessage(&msg, NULL, 0U, 0U, PM_NOREMOVE);
				while (WM_QUIT != msg.message )
				{
					bGotMsg = PeekMessage(&msg, NULL, 0U, 0U, PM_REMOVE);
					if (bGotMsg)
					{
						TranslateMessage(&msg);
						DispatchMessage(&msg);
					}
					else
					{
						WaitMessage();
					}
				}
			}
			else
			{
				::MessageBox(NULL,"ERROR: Can't open the SoundServer.","ERROR",MB_ICONEXCLAMATION);
			}
			delete pServer;
		}

		return 0;
}



